package eu.europa.ec.edelivery.smp.auth;

import eu.europa.ec.edelivery.security.utils.SecurityUtils;
import eu.europa.ec.edelivery.smp.logging.SMPLogger;
import eu.europa.ec.edelivery.smp.logging.SMPLoggerFactory;
import org.springframework.security.authentication.AbstractAuthenticationToken;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;

import java.util.Objects;

/**
 * UI login authentication token. The token is generated by SMPAuthenticationService and is supported by the SMPAuthenticationProviderForUI.
 * It is "distinguished" from UsernamePasswordAuthenticationToken, generated by basic authentication,
 * which is used for stateless web services invocation using the Personal Access Token credentials.
 *
 * @author Joze Rihtarsic
 * @since 4.2
 */
public class UILoginAuthenticationToken extends UsernamePasswordAuthenticationToken {
    private static final SMPLogger LOG = SMPLoggerFactory.getLogger(UILoginAuthenticationToken.class);
    SMPUserDetails userDetails;

    public UILoginAuthenticationToken(Object principal, Object credentials, SMPUserDetails userDetails) {
        super(principal, credentials, userDetails == null ? null : userDetails.getAuthorities());
        setDetails(userDetails);
        this.userDetails = userDetails;
    }

    public SecurityUtils.Secret getSecret() {

        if (userDetails == null) {
            LOG.warn("Can not retrieve security token for session. User details is null!");
            return null;
        }
        return userDetails.getSessionSecret();
    }

    public SMPUserDetails getUserDetails() {
        return userDetails;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof AbstractAuthenticationToken)) return false;
        if (!super.equals(o)) return false;
        UILoginAuthenticationToken that = (UILoginAuthenticationToken) o;
        // also check super equals (roles..) which is implemented in AbstractAuthenticationToken
        return Objects.equals(getDetails(), that.getDetails()) && super.equals(that);
    }

    @Override
    public int hashCode() {
        return Objects.hash(super.hashCode(), getDetails());
    }
}
